require 'spec_helper'
require 'books'
require 'pry'
require 'square_connector'
require 'securerandom'
require 'uri'

describe Books do
  it "should have a VERSION constant" do
    expect(subject.const_get('VERSION')).to_not be_empty
  end

  it "should create books and journal end-to-end" do
    books_host = "https://abishopric.local:32191"
    client_tls_cert = "/data/app/books/secrets/books-abishopric.local-s2s.crt"
    client_tls_key = "/data/app/books/secrets/books-abishopric.local-s2s.key"
    s2s_ca = "/data/app/books/secrets/service2service.ca.pem"
    client_signing_pubkey = "/Users/abishopric/books/books-abishopric-32-byte-private-key.pub"
    client_signing_privkey = "/Users/abishopric/books/books-abishopric-32-byte-private-key.priv"

    ################

    connector = SquareConnector::Client.new(
      services: {
        books: { url: URI.parse(books_host) },
      },
      certs: {
        client_cert: client_tls_cert,
        client_key: clinet_tls_key,
        ca_certificate: s2s_ca,
      },
      logger: Logger.new(STDERR),
      app_name: 'TestApp'
    )
    client = Books::StandardClient.new(connector: connector,
      books_authority_client: FakeAuthorityClient.new,
      eddsa_public_key_path: client_signing_pubkey,
      eddsa_private_key_path: client_signing_privkey)

    client.register_myself

    client.create_denomination({
      full_name: "United States Dollars",
      short_name: "USD"
    })

    owner = SecureRandom.hex

    asset_book = client.create_book({
      denomination: "USD",
      min_balance: 0,
      idempotence_token: SecureRandom.hex,
      contra: false,
      max_balance: 1000,
      type: "cash",
      owner: owner,
      category: Squareup::Books::Service::BookCategory::ASSET
    }).book

    expect(asset_book).to_not be_nil

    liability_book = client.create_book({
      denomination: "USD",
      max_balance: 0,
      idempotence_token: SecureRandom.hex,
      contra: false,
      min_balance: -1000,
      type: "cash",
      owner: owner,
      category: Squareup::Books::Service::BookCategory::LIABILITY
    }).book

    response = client.get_books_matching(filter: {
      :owner => owner
    })

    expect(response.books.length).to eq(2)

    journal_resp = client.journal({
      entries: [{
        to_amount: 100,
        from_amount: 0,
        book_id: asset_book.book_id,
        book_entry_type: "cash_received"
      }, {
        from_amount: 100,
        to_amount: 0,
        book_id: liability_book.book_id,
        book_entry_type: "cash_loaned"
      }],
      idempotence_token: SecureRandom.hex,
      journal_entry_type: "loan"
    })

    expect(journal_resp.status).to eq(Squareup::Books::Service::JournalStatus::JOURNAL_SUCCESS)
  end
end
